const { ImageFill, LinearGradient, RadialGradient, SymbolInstance, Shadow } = require('scenegraph')

function getVisibleInfo(node) {
    if (node.opacity < 0.01) {
        if (node instanceof SymbolInstance) {
            if (!node.isMaster) {
                return false;
            } else {    //Symbol master'da ne olursa olsun visible değiştirilmez.
                return true;
            }
        } else {
            return false;
        }
    } else {
        return node.visible;
    }
}

function getColorStopsAsRgba(colorStops) {
    try {
        colorStops.forEach(colorStop => {
            colorStop.color = colorStop.color.toRgba()
        });
    } catch (error) {
        console.log(error)
    }
}

class SceneNode {

    constructor(selection) {
        this.guid = selection.guid;
        this.parent = selection.parent;
        this.children = selection.children;
        this.isInArtworkTree = selection.isInArtworkTree;
        this.isContainer = selection.isContainer;
        this.selected = selection.selected;
        this.opacity = selection.opacity;
        this.visible = getVisibleInfo(selection);
        this.blendMode = selection.blendMode;
        this.transform = selection.transform;
        this.translation = selection.translation;
        this.rotation = selection.rotation;
        this.globalBounds = selection.globalBounds;
        this.localBounds = selection.localBounds;
        this.boundsInParent = selection.boundsInParent;
        this.topLeftInParent = selection.topLeftInParent;
        this.localCenterPoint = selection.localCenterPoint;
        this.globalDrawBounds = selection.globalDrawBounds;
        this.name = selection.name;
        this.hasDefaultName = selection.hasDefaultName;
        this.locked = selection.locked;
        this.markedForExport = selection.markedForExport;
        this.fixedWhenScrolling = selection.fixedWhenScrolling;
        this.triggeredInteractions = selection.triggeredInteractions;
        this.horizontalConstraints = selection.horizontalConstraints;
        this.verticalConstraints = selection.verticalConstraints;
        this.hasCustomConstraints = selection.hasCustomConstraints;
        this.hasLinkedContent = selection.hasLinkedContent;
        this.pluginData = selection.pluginData;
        this.sharedPluginData = selection.sharedPluginData;
    }
}

class GraphicNode {

    constructor(selection) {
        try {
            if (selection.fill instanceof ImageFill) {
                this.fill = "ImageFill"
            } else if (selection.fill instanceof LinearGradient) {
                let gradientFill = new LinearGradientModel(selection)
                getColorStopsAsRgba(gradientFill.colorStops)
                this.fill = gradientFill
            } else if (selection.fill instanceof RadialGradient) {
                let gradientFill = new RadialGradientModel(selection)
                getColorStopsAsRgba(gradientFill.colorStops)
                this.fill = gradientFill
            } else {
                this.fill = selection.fill.toRgba()
            }
        } catch{
            this.fill = selection.fill
        }
        this.fillEnabled = selection.fillEnabled;
        try {
            if (selection.stroke instanceof ImageFill) {
                this.stroke = "ImageFill"
            } else {
                this.stroke = selection.stroke.toRgba()
            }
        } catch{
            this.stroke = selection.stroke
        }
        this.strokeEnabled = selection.strokeEnabled;
        this.strokeWidth = selection.strokeWidth;
        this.strokePosition = selection.strokePosition;
        this.strokeEndCaps = selection.strokeEndCaps;
        this.strokeJoins = selection.strokeJoins;
        this.strokeMiterLimit = selection.strokeMiterLimit;
        this.strokeDashArray = selection.strokeDashArray;
        this.strokeDashOffset = selection.strokeDashOffset;
        try {
            if (selection.shadow instanceof Shadow) {
                this.shadow = new ShadowModel(selection)
            } else {
                this.shadow = selection.shadow
            }
        } catch{
            this.shadow = selection.shadow
        }
        this.blur = selection.blur;
        this.pathData = selection.pathData;
        this.hasLinkedGraphicFill = selection.hasLinkedGraphicFill;
    }
}

class RectangleModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.graphicNode = new GraphicNode(selection)
        this.nodeType = "Rectangle"
        this.width = selection.width
        this.height = selection.height
        this.cornerRadii = selection.cornerRadii
        this.hasRoundedCorners = selection.hasRoundedCorners
        this.effectiveCornerRadii = selection.effectiveCornerRadii
    }
}

class TextModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.graphicNode = new GraphicNode(selection)
        this.text = selection.text
        this.nodeType = "Text"
        this.styleRanges = selection.styleRanges
        this.fontFamily = selection.fontFamily
        this.fontStyle = selection.fontStyle
        this.fontSize = selection.fontSize
        this.charSpacing = selection.charSpacing
        this.underline = selection.underline
        this.strikethrough = selection.strikethrough
        this.textTransform = selection.textTransform
        this.textScript = selection.textScript
        this.flipY = selection.flipY
        this.textAlign = selection.textAlign
        this.lineSpacing = selection.lineSpacing
        this.paragraphSpacing = selection.paragraphSpacing
        this.areaBox = selection.areaBox
        this.clippedByArea = selection.clippedByArea
    }
}

class EllipseModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.graphicNode = new GraphicNode(selection)
        this.nodeType = "Ellipse"
        this.radiusX = selection.radiusX
        this.radiusY = selection.radiusY
        this.isCircle = selection.isCircle
    }
}

class PolygonModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.graphicNode = new GraphicNode(selection)
        this.nodeType = "Polygon"
        this.width = selection.width
        this.height = selection.height
        this.cornerCount = selection.cornerCount
        this.cornerRadii = selection.cornerRadii
        this.hasRoundedCorners = selection.hasRoundedCorners
        this.starRatio = selection.starRatio
    }
}

class LineModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection);
        this.graphicNode = new GraphicNode(selection);
        this.nodeType = "Line"
        this.start = selection.start
        this.end = selection.end
    }
}

class PathModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.graphicNode = new GraphicNode(selection)
        this.nodeType = "Path"
        this.pathData = selection.pathData
    }
}

class GroupModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.nodeType = "Group"
        this.dynamicLayout = selection.dynamicLayout
        this.mask = selection.mask
    }
}

class BooleanGroupModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.graphicNode = new GraphicNode(selection)
        this.nodeType = "BooleanGroup"
        this.pathOp = selection.pathOp
    }
}

class SymbolInstanceModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.nodeType = "SymbolInstance"
        this.symbolId = selection.symbolId
        this.isMaster = selection.isMaster
        this.dynamicLayout = selection.dynamicLayout
    }
}

class RepeatGridModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.nodeType = "RepeatGrid"
        this.width = selection.width
        this.height = selection.height
        this.numColumns = selection.numColumns
        this.numRows = selection.numRows
        this.paddingX = selection.paddingX
        this.paddingY = selection.paddingY
        this.cellSize = selection.cellSize
    }
}

class ScrollableGroupModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.nodeType = "ScrollableGroup"
        this.scrollingType = selection.scrollingType
        this.viewport = selection.viewport
    }
}

class ArtboardModel {

    constructor(selection) {
        this.sceneNode = new SceneNode(selection)
        this.graphicNode = new GraphicNode(selection)
        this.nodeType = "Artboard"
        this.width = selection.width
        this.height = selection.height
        this.viewportHeight = selection.viewportHeight
        this.incomingInteractions = selection.incomingInteractions
        this.isHomeArtboard = selection.isHomeArtboard
        this.dynamicLayout = selection.dynamicLayout
    }
}

class ShadowModel {

    constructor(selection) {
        this.x = selection.shadow.x
        this.y = selection.shadow.x
        this.blur = selection.shadow.blur
        this.color = selection.shadow.color.toRgba()
        this.visible = selection.shadow.visible
    }
}

class BaseGradientModel {

    constructor(selection) {
        this.endX = selection.fill.endX
        this.endY = selection.fill.endY
        this.startX = selection.fill.startX
        this.startY = selection.fill.startY
        this.colorStops = selection.fill.colorStops
    }
}

class LinearGradientModel extends BaseGradientModel {

    constructor(selection) {
        super(selection)
        this.type = 'linear' 
    }
}

class RadialGradientModel extends BaseGradientModel {

    constructor(selection) {
        super(selection)
        this.endR = selection.fill.endR
        this.startR = selection.fill.startR
        this.gradientTransform = selection.fill.gradientTransform
        this.type = 'radial' 
    }
}

module.exports = {
    RectangleModel,
    TextModel,
    EllipseModel,
    PolygonModel,
    LineModel,
    PathModel,
    GroupModel,
    BooleanGroupModel,
    SymbolInstanceModel,
    RepeatGridModel,
    ScrollableGroupModel,
    ArtboardModel,
    ShadowModel
}