const { alert, confirm, prompt, error, warning } = require("./lib/dialogs.js");
const { Artboard, RootNode } = require('scenegraph')

async function isAnyArtboardSelected(selection) {
    if (selection.items.length === 0) {
        return false
    } else {
        for (var i = 0; i < selection.items.length; i++) {
            var node = selection.items[i]
            while ((!(node instanceof Artboard)) && (!(node instanceof RootNode)) && node) {
                node = node.parent
            }
            if (node instanceof Artboard) {
                return true
            }
        }
    }
    return false
}

async function checkInternetConnection() {
    return navigator.onLine
}

function checkNodeHasExportable(node) {
    var isNodeHasExportable = false
    if (node.markedForExport) {
        isNodeHasExportable = true
        return isNodeHasExportable
    }
    if (node.children.length > 0) {
        node.children.forEach(function (childNode, i) {
            var isChildHasExportable = checkNodeHasExportable(childNode)
            if (isNodeHasExportable || isChildHasExportable) {
                isNodeHasExportable = true
                return isNodeHasExportable
            }
        });
    }
    return isNodeHasExportable
}

async function checkNonExportableArtboards(uniqueArtboards) {
    var nonExportableArtboards = []
    var errorCheck = false
    for (var i = 0; i < uniqueArtboards.length; i++) {
        var isNodeHasExportable = checkNodeHasExportable(uniqueArtboards[i])
        if (!isNodeHasExportable) {
            nonExportableArtboards.push(uniqueArtboards[i].name)
            errorCheck = true
        }
    }
    if (errorCheck) {
        var messages = ["Mark Icons for Export", "Mark icons/images/logos exportable so that they can be uploaded as images to Monday Hero. No need to mark everything for export such as texts or other shapes, it’s only needed for icons, images, or logos.",
            "Please note that if you like to get a part of the artboard as a whole image, you can also use this tip.", "@_@", ""]

        messages.push("", "", "", "", "## MORE INFORMATION", "-----", "[If you want to get the best results, here you can find the handoff rules.](https://learn.mondayhero.io/getting-started/design-handoff-to-ios-android-developer)")

        const feedback = await confirm(...messages)
        switch (feedback.which) {
            case 0:
                return true
            case 1:
                return false
        }
    } else {
        return false
    }
}

module.exports = {
    isAnyArtboardSelected,
    checkNonExportableArtboards,
    checkInternetConnection
}